using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Markup;
using Rhizome.Contract;
using Rhizome.Contract.Visitor;
using Rhizome.Internal;

namespace Rhizome.Model
{
    [ContentProperty("Actions")]
	public class AggregateAction<TPlant> : ActionBase<TPlant>, IForwardEvent
    {
        public AggregateAction(params IAction[] actions) : this()
        {
            foreach (var action in actions)
                Actions.Add(action);
        }

        public AggregateAction()
        {
            Actions = new RhizomeElementCollection<IAction>(this);
        }

        public Collection<IAction> Actions { get; private set; }

        public override void Process()
        {
            Actions.ForEach(action => action.Process());
        }

    	#region Implementation of IForwardEvent

    	object IForwardEvent.GetForwardEvent()
    	{
    		return Actions.GetForwardEvent();
    	}

    	#endregion

    	#region Overrides of RhizomeElement

    	protected override IEnumerable<IEnumerable> GetElementCollections()
    	{
    		return base.GetElementCollections().Concat(new[] {Actions});
    	}

    	#endregion
    }

    public sealed class AggregateAction : AggregateAction<object>
    {
    }
}