using System.Diagnostics;
using System.Windows.Markup;
using Rhizome.Contract;

namespace Rhizome.Model
{
    [ContentProperty("Action")]
    public class CriterionDecorator<TPlant> : ActionBase<TPlant>
    {
        private IAction _action;
        private ICriterion _criterion;

        public ICriterion Criterion
        {
            [DebuggerStepThrough]
            get { return _criterion; }
            set
            {
                if (_criterion == value) return;
                _criterion = value;
                var r = _criterion as RhizomeElement;
                if (r != null) r.Parent = this;
            }
        }

        /// <summary>
        ///   Inner action
        /// </summary>
        public IAction Action
        {
            [DebuggerStepThrough]
            get { return _action; }
            set
            {
                if (_action == value) return;
                _action = value;
                var r = _action as RhizomeElement;
                if (r != null) r.Parent = this;
            }
        }

        public override sealed void Process()
        {
            if (Action == null) throw new ValidationException("Action not set");

            if (Criterion == null || Criterion.Test())
                Action.Process();
        }
    }

    public sealed class CriterionDecorator : CriterionDecorator<object>
    {
    }
}