using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Markup;
using Rhizome.Contract;
using Rhizome.Contract.Visitor;
using Rhizome.Internal;

namespace Rhizome.Model
{
    [ContentProperty("Triggers")]
    public class MultiTrigger<TPlant> : TriggerBase<TPlant>, IForwardEvent
    {
        public MultiTrigger()
        {
            Triggers = new RhizomeElementCollection<ITrigger>(this);
        }

        public MultiTrigger(params ITrigger[] triggers) : this()
        {
            foreach (var trigger in triggers)
                Triggers.Add(trigger);
        }

        public Collection<ITrigger> Triggers { get; private set; }

        #region IForwardEvent Members

    	public sealed override object GetForwardEvent()
        {
        	return Triggers.GetForwardEvent();
        }

        #endregion

        public sealed override void Initialize()
        {
            Triggers.ForEachOfType<IStateInitialize>(t => t.Initialize());
        }

        public sealed override void Cleanup()
        {
            Triggers.ForEachOfType<IStateInitialize>(t => t.Cleanup());
        }

    	#region Overrides of RhizomeElement

    	protected override IEnumerable<IEnumerable> GetElementCollections()
    	{
    		return base.GetElementCollections().Concat(new[] {Triggers});
    	}

    	#endregion
    }

    public class MultiTrigger : MultiTrigger<object>
    {
    }
}